=begin
A fully functional VTK widget for Tkinter that uses
vtkGenericRenderWindowInteractor.  The widget is called
vtkTkRenderWindowInteractor.  The initialization part of this code is
similar to that of the vtkTkRenderWidget.

=end

require 'tk'
require 'vtk'
require 'vtk/tk/vtkLoadRubyTkWidgets'


class Vtk::TkRenderWindowInteractor < TkWindow
=begin
     A vtkTkRenderWidndowInteractor for Python.

    Use GetRenderWindow to get the vtkRenderWindow.

    Create with the keyword stereo=1 in order to generate a
    stereo-capable window.

    Create with the keyword focus_on_enter=1 to enable
    focus-follows-mouse.  The default is for a click-to-focus mode.    

    __getattr__ is used to make the widget also behave like a
    vtkGenericRenderWindowInteractor.
=end
  TkCommandNames = ['vtkTkRenderWidget'.freeze].freeze

  def initialize(master, kw={} )
=begin
        Constructor.

        Keyword arguments:

          rw -- Use passed render window instead of creating a new one.

          stereo -- If True, generate a stereo-capable window.
          Defaults to False.

          focus_on_enter -- If True, use a focus-follows-mouse mode.
          Defaults to False where the widget will use a click-to-focus
          mode.
=end
    # load the necessary extensions into tk
    Vtk.LoadRubyTkWidgets

    _symbolkey2str(kw)

    if kw['rw']
      renderWindow = kw['rw']
    else
      renderWindow = Vtk::RenderWindow.new
    end

    if kw['stereo']
      renderWindow.StereoCapableWindowOn
      kw.delete('stereo')
    end

    # check if focus should follow mouse
    if kw['focus_on_enter']
      @FocusOnEnter = true
      kw.delte('focus_on_enter')
    else
      @FocusOnEnter = false
    end

    kw['rw'] = renderWindow.GetAddressAsString("vtkRenderWindow")
    kw['widgetname'] = 'vtkTkRenderWindowInteractor'
#    kw['widgetname'] = 'vtkTkRenderWidget'
    super( master, kw )
    renderWindow.UnRegister(nil)

    @Iren = Vtk::GenericRenderWindowInteractor.new
    @Iren.SetRenderWindow(renderWindow)
    @Iren.UnRegister(nil)

    createTimer = Proc.new{|obj, evt|
      self.after(10, Proc.new{ @Iren.TimerEvent })
    }
    destroyTimer = Proc.new{|obj, event|
=begin
       The timer is a one shot timer so will expire automatically.
=end
      return 1
    }

    @Iren.AddObserver('CreateTimerEvent', createTimer)
    @Iren.AddObserver('DestroyTimerEvent', destroyTimer)



    @OldFocus = nil

    # private attributes
    @InExpose = false

    # create the Tk bindings
    self.BindEvents
    #self.tk_focusFollowsMouse

  end

  def BindEvents
=begin
        Bind all the events.
=end
    self.bind("Motion"){|e,s| s||s=self; s.MouseMoveEvent(e, 0, 0) }
    self.bind("Control-Motion"){|e,s| s||s=self; s.MouseMoveEvent(e, 1, 0) }
    self.bind("Shift-Motion"){|e,s| s||s=self; s.MouseMoveEvent(e, 1, 1) }
    self.bind("Control-Shift-Motion"){|e,s| s||s=self; s.MouseMoveEvent(e, 0, 1) }

    # Left Button
    self.bind("ButtonPress-1"){|e,s| s||s=self; s.LeftButtonPressEvent(e, 0, 0) }
    self.bind("Control-ButtonPress-1"){|e,s| s||s=self; s.LeftButtonPressEvent(e, 1, 0) }
    self.bind("Shift-ButtonPress-1"){|e,s| s||s=self; s.LeftButtonPressEvent(e, 0, 1) }
    self.bind("Control-Shift-ButtonPress-1"){|e,s| s||s=self; s.LeftButtonPressEvent(e, 1, 1) }
    self.bind("ButtonRelease-1"){|e,s| s||s=self; s.LeftButtonReleaseEvent(e, 0, 0) }
    self.bind("Control-ButtonRelease-1"){|e,s| s||s=self; s.LeftButtonReleaseEvent(e, 1, 0) }
    self.bind("Shift-ButtonRelease-1"){|e,s| s||s=self; s.LeftButtonReleaseEvent(e, 0, 1) }
    self.bind("Control-Shift-ButtonRelease-1"){|e,s| s||s=self; s.LeftButtonReleaseEvent(e, 1, 1) }

    # Middle Button
    self.bind("ButtonPress-2"){|e,s| s||s=self; s.MiddleButtonPressEvent(e, 0, 0) }
    self.bind("Control-ButtonPress-2"){|e,s| s||s=self; s.MiddleButtonPressEvent(e, 1, 0) }
    self.bind("Shift-ButtonPress-2"){|e,s| s||s=self; s.MiddleButtonPressEvent(e, 0, 1) }
    self.bind("Control-Shift-ButtonPress-2"){|e,s| s||s=self; s.MiddleButtonPressEvent(e, 1, 1) }
    self.bind("ButtonRelease-2"){|e,s| s||s=self; s.MiddleButtonReleaseEvent(e, 0, 0) }
    self.bind("Control-ButtonRelease-2"){|e,s| s||s=self; s.MiddleButtonReleaseEvent(e, 1, 0) }
    self.bind("Shift-ButtonRelease-2"){|e,s| s||s=self; s.MiddleButtonReleaseEvent(e, 0, 1) }
    self.bind("Control-Shift-ButtonRelease-2"){|e,s| s||s=self; s.MiddleButtonReleaseEvent(e, 1, 1) }

    # Right Button
    self.bind("ButtonPress-3"){|e,s| s||s=self; s.RightButtonPressEvent(e, 0, 0) }
    self.bind("Control-ButtonPress-3"){|e,s| s||s=self; s.RightButtonPressEvent(e, 1, 0) }
    self.bind("Shift-ButtonPress-3"){|e,s| s||s=self; s.RightButtonPressEvent(e, 0, 1) }
    self.bind("Control-Shift-ButtonPress-3"){|e,s| s||s=self; s.RightButtonPressEvent(e, 1, 1) }
    self.bind("ButtonRelease-3"){|e,s| s||s=self; s.RightButtonReleaseEvent(e, 0, 0) }
    self.bind("Control-ButtonRelease-3"){|e,s| s||s=self; s.RightButtonReleaseEvent(e, 1, 0) }
    self.bind("Shift-ButtonRelease-3"){|e,s| s||s=self; s.RightButtonReleaseEvent(e, 0, 1) }
    self.bind("Control-Shift-ButtonRelease-3"){|e,s| s||s=self; s.RightButtonReleaseEvent(e, 1, 1) }

    if Config::CONFIG['arch'] =~ /win32/
      self.bind("MouseWheel"){|e,s| s||s=self; s.MouseWheelEvent(e, 0, 0) }
      self.bind("Control-MouseWheel"){|e,s| s||s=self; s.MouseWheelEvent(e, 1, 0) }
      self.bind("Shift-MouseWheel"){|e,s| s||s=self; s.MouseWheelEvent(e, 0, 1) }
      self.bind("Control-Shift-MouseWheel"){|e,s| s||s=self; s.MouseWheelEvent(e, 1, 1) }
    else
      # Mouse wheel forward event 
      self.bind("ButtonPress-4"){|e,s| s||s=self; s.MouseWheelForwardEvent(e, 0, 0) }
      self.bind("Control-ButtonPress-4"){|e,s| s||s=self; s.MouseWheelForwardEvent(e, 1, 0) }
      self.bind("Shift-ButtonPress-4"){|e,s| s||s=self; s.MouseWheelForwardEvent(e, 0, 1) }
      self.bind("Control-Shift-ButtonPress-4"){|e,s| s||s=self; s.MouseWheelForwardEvent(e, 1, 1) }

      # Mouse wheel backward event 
      self.bind("ButtonPress-5"){|e,s| s||s=self; s.MouseWheelBackwardEvent(e, 0, 0) }
      self.bind("Control-ButtonPress-5"){|e,s| s||s=self; s.MouseWheelBackwardEvent(e, 1, 0) }
      self.bind("Shift-ButtonPress-5"){|e,s| s||s=self; s.MouseWheelBackwardEvent(e, 0, 1) }
      self.bind("Control-Shift-ButtonPress-5"){|e,s| s||s=self; s.MouseWheelBackwardEvent(e, 1, 1) }
    end

    # Key related events
    self.bind("KeyPress"){|e,s| s||s=self; s.KeyPressEvent(e, 0, 0) }
    self.bind("Control-KeyPress"){|e,s| s||s=self; s.KeyPressEvent(e, 1, 0) }
    self.bind("Shift-KeyPress"){|e,s| s||s=self; s.KeyPressEvent(e, 0, 1) }
    self.bind("Control-Shift-KeyPress"){|e,s| s||s=self; s.KeyPressEvent(e, 1, 1) }

    self.bind("KeyRelease"){|e,s| s||s=self; s.KeyReleaseEvent(e, 0, 0) }
    self.bind("Control-KeyRelease"){|e,s| s||s=self; s.KeyReleaseEvent(e, 1, 0) }
    self.bind("Shift-KeyRelease"){|e,s| s||s=self; s.KeyReleaseEvent(e, 0, 1) }
    self.bind("Control-Shift-KeyRelease"){|e,s| s||s=self; s.KeyReleaseEvent(e, 1, 1) }

    self.bind("Enter"){|e,s| s||s=self; s.EnterEvent(e, 0, 0) }
    self.bind("Control-Enter"){|e,s| s||s=self; s.EnterEvent(e, 1, 0) }
    self.bind("Shift-Enter"){|e,s| s||s=self; s.EnterEvent(e, 0, 1) }
    self.bind("Control-Shift-Enter"){|e,s| s||s=self; s.EnterEvent(e, 1, 1) }
    self.bind("Leave"){|e,s| s||s=self; s.LeaveEvent(e, 0, 0) }
    self.bind("Control-Leave"){|e,s| s||s=self; s.LeaveEvent(e, 1, 0) }
    self.bind("Shift-Leave"){|e,s| s||s=self; s.LeaveEvent(e, 0, 1) }
    self.bind("Control-Shift-Leave"){|e,s| s||s=self; s.LeaveEvent(e, 1, 1) }

    self.bind("Configure"){|e,s| s||s=self; s.ConfigureEvent(e) }
    self.bind("Expose"){|e,s| s||s=self; s.ExposeEvent }
  end

  def _GrabFocus(enter=0)
    @OldFocus=self.focus
    self.focus
  end

  def MouseMoveEvent(event, ctrl, shift)
    @Iren.SetEventInformationFlipY(event.x, event.y, ctrl,
                                   shift, 0.chr, 0, nil)
    @Iren.MouseMoveEvent
  end

  def LeftButtonPressEvent(event, ctrl, shift)
    @Iren.SetEventInformationFlipY(event.x, event.y, ctrl,
                                   shift, 0.chr, 0, nil)
    @Iren.LeftButtonPressEvent
    if  !@FocusOnEnter
      _GrabFocus
    end
  end

  def LeftButtonReleaseEvent(event, ctrl, shift)
    @Iren.SetEventInformationFlipY(event.x, event.y, ctrl,
                                   shift, 0.chr, 0, nil)
    @Iren.LeftButtonReleaseEvent
  end

  def MiddleButtonPressEvent(event, ctrl, shift)
    @Iren.SetEventInformationFlipY(event.x, event.y, ctrl,
                                   shift, 0.chr, 0, nil)
    @Iren.MiddleButtonPressEvent
    if  !@FocusOnEnter
      _GrabFocus
    end
  end

  def MiddleButtonReleaseEvent( event, ctrl, shift)
    @Iren.SetEventInformationFlipY(event.x, event.y, ctrl,
                                   shift, 0.chr, 0, nil)
    @Iren.MiddleButtonReleaseEvent
  end

  def RightButtonPressEvent(event, ctrl, shift)
    @Iren.SetEventInformationFlipY(event.x, event.y, ctrl,
                                   shift, 0.chr, 0, nil)
    @Iren.RightButtonPressEvent
    if  !@FocusOnEnter
      _GrabFocus
    end
  end

  def RightButtonReleaseEvent(event, ctrl, shift)
    @Iren.SetEventInformationFlipY(event.x, event.y, ctrl,
                                   shift, 0.chr, 0, nil)
    @Iren.RightButtonReleaseEvent
  end

  def MouseWheelEvent(event, ctrl, shift)
    @Iren.SetEventInformationFlipY(event.x, event.y, ctrl,
                                   shift, 0.chr, 0, nil)
    if event.delta > 0
      @Iren.MouseWheelForwardEvent
    else
      @Iren.MouseWheelBackwardEvent
    end
  end

  def MouseWheelForwardEvent(event, ctrl, shift)
    @Iren.SetEventInformationFlipY(event.x, event.y, ctrl,
                                   shift, 0.chr, 0, nil)
    @Iren.MouseWheelForwardEvent
  end

  def MouseWheelBackwardEvent(event, ctrl, shift)
    @Iren.SetEventInformationFlipY(event.x, event.y, ctrl,
                                   shift, 0.chr, 0, nil)
    @Iren.MouseWheelBackwardEvent
  end

  def KeyPressEvent(event, ctrl, shift)
    key = 0.chr
    if event.keysym_num < 256
      key = event.keysym_num.chr
    end
    @Iren.SetEventInformationFlipY(event.x, event.y, ctrl,
                                   shift, key, 0, event.keysym)
    @Iren.KeyPressEvent
    @Iren.CharEvent
  end

  def KeyReleaseEvent(event, ctrl, shift)
    key = 0.chr
    if event.keysym_num < 256
      key = event.keysym_num.chr
    end
    @Iren.SetEventInformationFlipY(event.x, event.y, ctrl,
                                   shift, key, 0, event.keysym)
    @Iren.KeyReleaseEvent
  end

  def ConfigureEvent(event)
    @Iren.SetSize(event.width, event.height)
    @Iren.ConfigureEvent
  end

  def EnterEvent(event, ctrl, shift)
    if @FocusOnEnter
      _GrabFocus
    end
    @Iren.SetEventInformationFlipY(event.x, event.y, ctrl, shift,
                                   0.chr, 0, nil)
    @Iren.EnterEvent
  end

  def LeaveEvent(event, ctrl, shift)
    if @FocusOnEnter && (@OldFocus != nil)
      @OldFocus.focus
    end
    @Iren.SetEventInformationFlipY(event.x, event.y, ctrl, shift,
                                   0.chr, 0, nil)
    @Iren.LeaveEvent
  end

  def ExposeEvent
    if !@InExpose
      @InExpose = true
      self.update
      self.GetRenderWindow.Render
      @InExpose = false
    end
  end

  def GetRenderWindow
    addr = Tk.tk_call(self, 'GetRenderWindow')[5..-1]
    renderWindow = Vtk::RenderWindow.new('_%s_vtkRenderWindow_p' % addr)
    renderWindow.UnRegister(nil)
    return renderWindow
  end

  def Render
    self.GetRenderWindow.Render
  end

  # wrapping methods of Vtk::RenderWindowInteractor
  names = self.instance_methods
  Vtk::RenderWindowInteractor.instance_methods.each{|name|
    unless names.include?(name)
      argc = Vtk::RenderWindowInteractor.instance_method(name).arity
      argv = []
      if argc >= 0
        for i in 0...argc
          argv.push "arg#{i}"
        end
      else
        for i in 0...-argc-1
          argv.push "arg#{i}"
        end
        argv.push "*optional"
      end
      eval <<-"EOF"
      def #{name}( #{argv.join(",")} )
        @Iren.#{name}( #{argv.join(",")} )
      end
      EOF
    end
  }
end


#----------------------------------------------------------------------------  
def vtkRenderWindowInteractorConeExample
=begin
    Like it says, just a simple example
=end
  # create root window
  root = TkRoot.new

  # create vtkTkRenderWidget
  pane = Vtk::TkRenderWindowInteractor.new(root, 'width'=>300, 'height'=>300)
  pane.Initialize

  pane.AddObserver("ExitEvent", Proc.new{|o,e,q| exit })

  ren = Vtk::Renderer.new
  pane.GetRenderWindow.AddRenderer(ren)

  cone = Vtk::ConeSource.new
  cone.SetResolution(8)

  coneMapper = Vtk::PolyDataMapper.new
  coneMapper.SetInput(cone.GetOutput)

  coneActor = Vtk::Actor.new
  coneActor.SetMapper(coneMapper)

  ren.AddActor(coneActor)

  # pack the pane into the tk root
  pane.pack('fill'=>'both', 'expand'=>1)
  pane.Start

  # start the tk mainloop
  root.mainloop
end

if $0 == __FILE__
  vtkRenderWindowInteractorConeExample
end
