begin
  require "narray"

  module Vtk
    class ImageExportToArray
      def initialize
        @export = ImageExport.new
        @ConvertUnsignedShortToInt = false
      end

      @@typeDict = { VTK_CHAR=>NArray::BYTE,
                     VTK_UNSIGNED_CHAR=>NArray::BYTE,
                     VTK_SHORT=>NArray::SINT,
                     VTK_UNSIGNED_SHORT=>NArray::SINT,
                     VTK_INT=>NArray::INT,
                     VTK_FLOAT=>NArray::SFLOAT,
                     VTK_DOUBLE=>NArray::FLOAT }

      @@sizeDict = { VTK_CHAR=>1,
                     VTK_UNSIGNED_CHAR=>1,
                     VTK_SHORT=>2,
                     VTK_UNSIGNED_SHORT=>2,
                     VTK_INT=>4,
                     VTK_FLOAT=>4,
                     VTK_DOUBLE=>8 }

      # convert unsigned shorts to ints, to avoid sign problems
      def SetConvertUnsignedShortToInt(yesno)
        @ConvertUnsignedShortToInt = yesno
      end

      def GetConvertUnsignedShortToInt
        return @ConvertUnsignedShortToInt
      end
    
      def ConvertUnsignedShortToIntOn
        @ConvertUnsignedShortToInt = true
      end

      def ConvertUnsignedShortToIntOff
        @ConvertUnsignedShortToInt = false
      end

      # set the input
      def SetInput(input)
        return @export.SetInput(input)
      end

      def GetInput
        return @export.GetInput
      end

      def GetArray
        input = @export.GetInput
        input.UpdateInformation
        type = input.GetScalarType
        extent = input.GetWholeExtent
        numComponents = input.GetNumberOfScalarComponents
        dim = [extent[5]-extent[4]+1,
               extent[3]-extent[2]+1,
               extent[1]-extent[0]+1]
        if (numComponents > 1)
          dim = dim + [numComponents]
        end
        size = dim[0]*dim[1]*dim[2]*numComponents*@@sizeDict[type]

        imString = NArray.byte(size).to_s
        @export.Export(imString)
        imArray = NArray.to_na(imString,@@typeDict[type])

        # reshape array appropriately.
        imArray.reshape!(*dim)
        # convert unsigned short to int to avoid sign issues
        if (type == VTK_UNSIGNED_SHORT && @ConvertUnsignedShortToInt)
          imArray = umath.bitwise_and(imArray.astype(Numeric.Int32),0xffff)
          return imArray
        end
      end
        
      def GetDataExtent
        return @export.GetDataExtent
      end

      def GetDataSpacing
        return @export.GetDataSpacing
      end

      def GetDataOrigin
        return @export.GetDataOrigin
      end

    end
  end

rescue LoadError
end
