begin
  require 'narray'

  module Vtk
    class ImageImportFromArray
      def initialize
        @import = ImageImport.new
        @ConvertIntToUnsignedShort = false
        @Array = nil
      end

      @@typeDict = {NArray::BYTE=>VTK_UNSIGNED_CHAR,
                    NArray::SINT=>VTK_SHORT,
                    NArray::INT=>VTK_INT,
                    NArray::SFLOAT=>VTK_FLOAT,
                    NArray::FLOAT=>VTK_DOUBLE }

      @@sizeDict = { VTK_CHAR=>1,
                     VTK_UNSIGNED_CHAR=>1,
                     VTK_SHORT=>2,
                     VTK_UNSIGNED_SHORT=>2,
                     VTK_INT=>4,
                     VTK_FLOAT=>4,
                     VTK_DOUBLE=>8 }

      # convert 'Int32' to 'unsigned short'
      def SetConvertIntToUnsignedShort(yesno)
        @ConvertIntToUnsignedShort = yesno
      end

      def GetConvertIntToUnsignedShort
        return @ConvertIntToUnsignedShort
      end

      def ConvertIntToUnsignedShortOn
        @ConvertIntToUnsignedShort = true
      end

      def ConvertIntToUnsignedShortOff
        @ConvertIntToUnsignedShort = false
      end

      # get the output
      def GetOutput
        return @import.GetOutput
      end

      # import an array
      def SetArray(imArray)
        @Array = imArray
        numComponents = 1
        dim = imArray.shape

        if (dim.length == 4)
          numComponents = dim[3]
          dim = [dim[0],dim[1],dim[2]]
        end
            
        type = @@typeDict[imArray.typecode]

        if (@ConvertIntToUnsignedShort && imArray.typecode == NArray::INT)
            imTmpArr = imArray.astype(Numeric.Int16).to_s
            type = VTK_UNSIGNED_SHORT
        else
            imTmpArr = imArray.to_s
        end

        size = imTmpArr.len*@@sizeDict[type]
        @import.CopyImportVoidPointer(imTmpArr, size)
        @import.SetDataScalarType(type)
        @import.SetNumberOfScalarComponents(numComponents)
        extent = @import.GetDataExtent
        @import.SetDataExtent(extent[0],extent[0]+dim[2]-1,
                              extent[2],extent[2]+dim[1]-1,
                              extent[4],extent[4]+dim[0]-1)
        @import.SetWholeExtent(extent[0],extent[0]+dim[2]-1,
                               extent[2],extent[2]+dim[1]-1,
                               extent[4],extent[4]+dim[0]-1)
      end

      def GetArray
        return @Array
      end
        
      # a whole bunch of methods copied from vtkImageImport

      def SetDataExtent(extent)
        @import.SetDataExtent(extent)
      end

      def GetDataExtent
        return @import.GetDataExtent
      end
    
      def SetDataSpacing(spacing)
        @import.SetDataSpacing(spacing)
      end

      def GetDataSpacing
        return @import.GetDataSpacing
      end
    
      def SetDataOrigin(origin)
        @import.SetDataOrigin(origin)
      end

      def GetDataOrigin
        return @import.GetDataOrigin
      end
    end
  end

rescue LoadError
end

    
